import 'package:gainer_crypto/crypto_advanced.dart';

void main() {
  const inputText = "Hello GaiNeR";

  // HMAC Examples where you need a key
  const key = "Password";

  final listCryptoTypes = [GRCryptoTypeHMAC.hmacSHA1, GRCryptoTypeHMAC.hmacMD5];

  // Encrypt the input string sequentially the list of types provided and
  // return Hash String
  final hashString = grAdvHMACHashString(listCryptoTypes, inputText, key);
  print(hashString);

  // Encrypt the input string sequentially the list of types provided and
  // return Hash Bytes in terms of List<int>
  final hashBytes = grAdvHMACHashBytes(listCryptoTypes, inputText, key);
  print(hashBytes);

  const input1 = "Hello";
  const input2 = "GaiNeR";
  final chunks = [input1, input2];

  // Encrypt the chunks input string sequentially the list of types provided
  // and return Hash String
  final hashStringCh =
      grAdvHMACHashStringFromArray(listCryptoTypes, chunks, key);
  print(hashStringCh);

  // Encrypt the chunks input string sequentially the list of types provided
  // and return Hash Bytes in terms of List<int>
  final hashBytesCH = grAdvHMACHashBytesFromArray(listCryptoTypes, chunks, key);
  print(hashBytesCH);

  // Encrypt the input string multiple times of CryptoType provided and
  // return Hash String. Here 3 means repeat 3 times, which will actually
  // encrypt 4 times.
  final sha1HashWR =
      grHMACHashStringWR(GRCryptoTypeHMAC.hmacSHA1, 3, inputText, key);
  print(sha1HashWR);

  // Encrypt the input string multiple times of CryptoType provided and
  // return Hash Bytes in terms of List<int>. Here 2 means repeat 2 times,
  // which will actually encrypt 3 times.
  final md5BytesWR =
      grHMACHashBytesWR(GRCryptoTypeHMAC.hmacMD5, 2, inputText, key);
  print(md5BytesWR);

  // Encrypt the chunks input string multiple times of CryptoType provided and
  // return Hash String. Here 3 means repeat 3 times, which will actually
  // encrypt 4 times.
  final hashChunkWR =
      grHMACHashStringFromArrayWR(GRCryptoTypeHMAC.hmacSHA1, 3, chunks, key);
  print(hashChunkWR);

  // Encrypt the chunks input string multiple times of CryptoType provided and
  // return Hash Bytes in terms of List<int>. Here 2 means repeat 2 times,
  // which will actually encrypt 3 times.
  final bytesChunkWR =
      grHMACHashBytesFromArrayWR(GRCryptoTypeHMAC.hmacMD5, 2, chunks, key);
  print(bytesChunkWR);
}
