// Copyright (c) 2022, Pritam Hazra.

import 'crypto_types.dart';
import 'hash.dart';

String grAdvHashString(List<GRCryptoType> types, String input) {
  if (types.isEmpty) {
    return "";
  }
  var output = input;
  for (GRCryptoType type in types) {
    output = grHashString(type, output);
  }
  return output;
}

List<int> grAdvHashBytes(List<GRCryptoType> types, String input) {
  if (types.isEmpty) {
    return [];
  }
  var output = input;
  for (int i = 0; i < types.length; i++) {
    if (i == types.length - 1) {
      return grHashBytes(types[i], output);
    }
    output = grHashString(types[i], output);
  }
  return [];
}

String grAdvHashStringFromArray(List<GRCryptoType> types, List<String> input) {
  var output = "";
  if (types.isEmpty) {
    return output;
  }
  if (types.length == 1) {
    return grHashStringFromArray(types[0], input);
  }
  for (int i = 0; i < types.length; i++) {
    if (i == 0) {
      output = grHashStringFromArray(types[i], input);
    } else {
      output = grHashString(types[i], output);
    }
  }
  return output;
}

List<int> grAdvHashBytesFromArray(
    List<GRCryptoType> types, List<String> input) {
  var output = "";
  if (types.isEmpty) {
    return [];
  }
  if (types.length == 1) {
    return grHashBytesFromArray(types[0], input);
  }
  for (int i = 0; i < types.length; i++) {
    if (i == 0) {
      output = grHashStringFromArray(types[i], input);
    } else if (i == types.length - 1) {
      return grHashBytes(types[i], output);
    } else {
      output = grHashString(types[i], output);
    }
  }
  return [];
}

String grHashStringWR(GRCryptoType type, int repeat, String input) {
  if (repeat < 0) {
    return "";
  }
  var output = input;
  for (int i = 0; i <= repeat; i++) {
    output = grHashString(type, output);
  }
  return output;
}

List<int> grHashBytesWR(GRCryptoType type, int repeat, String input) {
  if (repeat < 0) {
    return [];
  }
  var output = input;
  for (int i = 0; i <= repeat; i++) {
    if (i == repeat) {
      return grHashBytes(type, output);
    }
    output = grHashString(type, output);
  }
  return [];
}

String grHashStringFromArrayWR(
    GRCryptoType type, int repeat, List<String> input) {
  var output = "";
  if (repeat < 0) {
    return output;
  }
  if (repeat == 0) {
    return grHashStringFromArray(type, input);
  }
  for (int i = 0; i <= repeat; i++) {
    if (i == 0) {
      output = grHashStringFromArray(type, input);
    } else {
      output = grHashString(type, output);
    }
  }
  return output;
}

List<int> grHashBytesFromArrayWR(
    GRCryptoType type, int repeat, List<String> input) {
  var output = "";
  if (repeat < 0) {
    return [];
  }
  if (repeat == 0) {
    return grHashBytesFromArray(type, input);
  }
  for (int i = 0; i <= repeat; i++) {
    if (i == 0) {
      output = grHashStringFromArray(type, input);
    } else if (i == repeat) {
      return grHashBytes(type, output);
    } else {
      output = grHashString(type, output);
    }
  }
  return [];
}
