// Copyright (c) 2022, Pritam Hazra.

import 'crypto_types.dart';
import 'hmac_hash.dart';

String grAdvHMACHashString(
    List<GRCryptoTypeHMAC> types, String input, String key) {
  if (types.isEmpty) {
    return "";
  }
  var output = input;
  for (GRCryptoTypeHMAC type in types) {
    output = grHMACHashString(type, output, key);
  }
  return output;
}

List<int> grAdvHMACHashBytes(
    List<GRCryptoTypeHMAC> types, String input, String key) {
  if (types.isEmpty) {
    return [];
  }
  var output = input;
  for (int i = 0; i < types.length; i++) {
    if (i == types.length - 1) {
      return grHMACHashBytes(types[i], output, key);
    }
    output = grHMACHashString(types[i], output, key);
  }
  return [];
}

String grAdvHMACHashStringFromArray(
    List<GRCryptoTypeHMAC> types, List<String> input, String key) {
  var output = "";
  if (types.isEmpty) {
    return output;
  }
  if (types.length == 1) {
    return grHMACHashStringFromArray(types[0], input, key);
  }
  for (int i = 0; i < types.length; i++) {
    if (i == 0) {
      output = grHMACHashStringFromArray(types[i], input, key);
    } else {
      output = grHMACHashString(types[i], output, key);
    }
  }
  return output;
}

List<int> grAdvHMACHashBytesFromArray(
    List<GRCryptoTypeHMAC> types, List<String> input, String key) {
  var output = "";
  if (types.isEmpty) {
    return [];
  }
  if (types.length == 1) {
    return grHMACHashBytesFromArray(types[0], input, key);
  }
  for (int i = 0; i < types.length; i++) {
    if (i == 0) {
      output = grHMACHashStringFromArray(types[i], input, key);
    } else if (i == types.length - 1) {
      return grHMACHashBytes(types[i], output, key);
    } else {
      output = grHMACHashString(types[i], output, key);
    }
  }
  return [];
}

String grHMACHashStringWR(
    GRCryptoTypeHMAC type, int repeat, String input, String key) {
  if (repeat < 0) {
    return "";
  }
  var output = input;
  for (int i = 0; i <= repeat; i++) {
    output = grHMACHashString(type, output, key);
  }
  return output;
}

List<int> grHMACHashBytesWR(
    GRCryptoTypeHMAC type, int repeat, String input, String key) {
  if (repeat < 0) {
    return [];
  }
  var output = input;
  for (int i = 0; i <= repeat; i++) {
    if (i == repeat) {
      return grHMACHashBytes(type, output, key);
    }
    output = grHMACHashString(type, output, key);
  }
  return [];
}

String grHMACHashStringFromArrayWR(
    GRCryptoTypeHMAC type, int repeat, List<String> input, String key) {
  var output = "";
  if (repeat < 0) {
    return output;
  }
  if (repeat == 0) {
    return grHMACHashStringFromArray(type, input, key);
  }
  for (int i = 0; i <= repeat; i++) {
    if (i == 0) {
      output = grHMACHashStringFromArray(type, input, key);
    } else {
      output = grHMACHashString(type, output, key);
    }
  }
  return output;
}

List<int> grHMACHashBytesFromArrayWR(
    GRCryptoTypeHMAC type, int repeat, List<String> input, String key) {
  var output = "";
  if (repeat < 0) {
    return [];
  }
  if (repeat == 0) {
    return grHMACHashBytesFromArray(type, input, key);
  }
  for (int i = 0; i <= repeat; i++) {
    if (i == 0) {
      output = grHMACHashStringFromArray(type, input, key);
    } else if (i == repeat) {
      return grHMACHashBytes(type, output, key);
    } else {
      output = grHMACHashString(type, output, key);
    }
  }
  return [];
}
