// Copyright (c) 2022, Pritam Hazra.

import 'crypto_types.dart';
import 'hash/hash_string.dart';
import 'hash/hash_bytes.dart';

/// This method takes two parameters,
///
/// • type: This should be of type GRCryptoType
///
/// • input: This should be of type String
///
/// Returns: A value of type String
///
/// Usage: Encrypts the input String value to a specific CryptoType and return
/// as a String. This is a one-sided encryption and can't be decrypted.
///
/// Example: To covert "Hello GaiNeR" to SHA224, we have to write like this,
///
/// String hashValue = grHashString(CryptoTypes.sha224, "Hello GaiNeR");
String grHashString(GRCryptoType type, String input) {
  return getHash(input, type);
}

/// This method takes two parameters,
///
/// • type: This should be of type GRCryptoType
///
/// • input: This should be of type String
///
/// Returns: A List of type int
///
/// Usage: Encrypts the input String value to a specific CryptoType and return
/// bytes stream as List<int>. This is a one-sided encryption and can't be decrypted.
///
/// Example: To covert "Hello GaiNeR" to SHA224, we have to write like this,
///
/// List<int> hashBytes = grHashBytes(CryptoTypes.sha224, "Hello GaiNeR");
List<int> grHashBytes(GRCryptoType type, String input) {
  return getHashBytes(input, type);
}

/// This method takes two parameters,
///
/// • type: This should be of type GRCryptoType
///
/// • input: This should be of type List<String>
///
/// Returns: A value of type String
///
/// Usage: Encrypts the Chunk of Strings to a specific CryptoType and return
/// as a String. The chunks should be passed as List<String>.
/// This is a one-sided encryption and can't be decrypted.
///
/// Example: To covert ["Hello", "GaiNeR"] to SHA224, we have to write like this,
///
/// String hashValue = grHashStringFromArray(CryptoTypes.sha224, ["Hello", "GaiNeR"]);
String grHashStringFromArray(GRCryptoType type, List<String> input) {
  return getHashFromStringArray(input, type);
}

/// This method takes two parameters,
///
/// • type: This should be of type GRCryptoType
///
/// • input: This should be of type List<String>
///
/// Returns: A List of type int
///
/// Usage: Encrypts the Chunk of Strings to a specific CryptoType and return
/// bytes stream as List<int>. The chunks should be passed as List<String>.
/// This is a one-sided encryption and can't be decrypted.
///
/// Example: To covert ["Hello", "GaiNeR"] to SHA224, we have to write like this,
///
/// List<int> hashBytes = grHashBytesFromArray(CryptoTypes.sha224, ["Hello", "GaiNeR"]);
List<int> grHashBytesFromArray(GRCryptoType type, List<String> input) {
  return getHashBytesFromStringArray(input, type);
}
