// Copyright (c) 2022, Pritam Hazra.
import 'dart:convert';
import 'package:convert/convert.dart';
import 'package:crypto/crypto.dart';
import 'pointycastle_converter.dart';
import '../crypto_types.dart';
import 'hash_types.dart';

List<int> getHashBytes(String from, GRCryptoType type) {
  final bytes = utf8.encode(from);
  final hash = getHashFor(type);
  if (hash != null) {
    final digest = hash.convert(bytes);
    return digest.bytes;
  }
  final hashConv = hashConverter(type);
  if (hashConv == null) {
    return [];
  }
  final hashBytes = hashConv.convert(bytes);
  return hashBytes;
}

List<int> getHashBytesFromStringArray(List<String> input, GRCryptoType type) {
  var chunks = [];
  for (String value in input) {
    chunks.add(utf8.encode(value));
  }
  var output = AccumulatorSink<Digest>();
  final hash = getHashFor(type);
  if (hash != null) {
    var chunkInput = hash.startChunkedConversion(output);
    for (List<int> chunk in chunks) {
      chunkInput.add(chunk);
    }
    chunkInput.close();
    final digest = output.events.single;
    return digest.bytes;
  }
  final hashConv = hashConverter(type);
  if (hashConv == null) {
    return [];
  }
  final hashBytes = convertFrom(input, hashConv);
  final digest = Digest(hashBytes);
  return digest.bytes;
}
