// Copyright (c) 2022, Pritam Hazra.

import 'crypto_types.dart';
import 'hmac_hash/hmac_hash_string.dart';
import 'hmac_hash/hmac_hash_bytes.dart';

/// This method takes two parameters,
///
/// • type: This should be of type GRCryptoType
///
/// • input: This should be of type String
///
/// • key: This should be of type String
///
/// Returns: A value of type String
///
/// Usage: Encrypts the input String value to a specific CryptoType with a key
/// and return as a String. This is a one-sided encryption and can't be decrypted.
///
/// Example: To covert "Hello GaiNeR" to SHA224 with key "Password", we have to write like this,
///
/// String hashValue = grHMACHashString(CryptoTypes.sha224, "Hello GaiNeR", "Password");
String grHMACHashString(GRCryptoTypeHMAC type, String input, String key) {
  return getHash(input, key, type);
}

/// This method takes two parameters,
///
/// • type: This should be of type GRCryptoType
///
/// • input: This should be of type String
///
/// • key: This should be of type String
///
/// Returns: A List of type int
///
/// Usage: Encrypts the input String value to a specific CryptoType with a key
/// and return bytes stream as List<int>. This is a one-sided encryption and
/// can't be decrypted.
///
/// Example: To covert "Hello GaiNeR" to SHA224 with key "Password", we have to write like this,
///
/// List<int> hashBytes = grHashBytes(CryptoTypes.sha224, "Hello GaiNeR", "Password");
List<int> grHMACHashBytes(GRCryptoTypeHMAC type, String input, String key) {
  return getHashBytes(input, key, type);
}

/// This method takes two parameters,
///
/// • type: This should be of type GRCryptoType
///
/// • input: This should be of type List<String>
///
/// • key: This should be of type String
///
/// Returns: A value of type String
///
/// Usage: Encrypts the Chunk of Strings to a specific CryptoType with a key
/// and return as a String. The chunks should be passed as List<String>.
/// This is a one-sided encryption and can't be decrypted.
///
/// Example: To covert ["Hello", "GaiNeR"] to SHA224 with key "Password",
/// we have to write like this,
///
/// String hashValue = grHashStringFromArray(CryptoTypes.sha224, ["Hello", "GaiNeR"], "Password");
String grHMACHashStringFromArray(
    GRCryptoTypeHMAC type, List<String> input, String key) {
  return getHashFromStringArray(input, key, type);
}

/// This method takes two parameters,
///
/// • type: This should be of type GRCryptoType
///
/// • input: This should be of type List<String>
///
/// • key: This should be of type String
///
/// Returns: A List of type int
///
/// Usage: Encrypts the Chunk of Strings to a specific CryptoType with a key and
/// return bytes stream as List<int>. The chunks should be passed as List<String>.
/// This is a one-sided encryption and can't be decrypted.
///
/// Example: To covert ["Hello", "GaiNeR"] to SHA224 with key "Password",
/// we have to write like this,
///
/// List<int> hashBytes = grHashBytesFromArray(CryptoTypes.sha224, ["Hello", "GaiNeR"], "Password");
List<int> grHMACHashBytesFromArray(
    GRCryptoTypeHMAC type, List<String> input, String key) {
  return getHashBytesFromStringArray(input, key, type);
}
