// Copyright (c) 2022, Pritam Hazra.

import 'dart:convert';
import 'package:convert/convert.dart';
import 'package:crypto/crypto.dart';
import '../crypto_types.dart';
import 'hmac_hash_types.dart';

String getHash(String from, String key, GRCryptoTypeHMAC type) {
  final keyBytes = utf8.encode(key);
  final bytes = utf8.encode(from);
  final hmac = Hmac(getHashFor(type), keyBytes);
  final digest = hmac.convert(bytes);
  return "$digest";
}

String getHashFromStringArray(
    List<String> input, String key, GRCryptoTypeHMAC type) {
  var chunks = [];
  for (String value in input) {
    chunks.add(utf8.encode(value));
  }
  var output = AccumulatorSink<Digest>();
  final keyBytes = utf8.encode(key);
  final hmac = Hmac(getHashFor(type), keyBytes);
  var chunkInput = hmac.startChunkedConversion(output);
  for (List<int> chunk in chunks) {
    chunkInput.add(chunk);
  }
  chunkInput.close();
  final digest = output.events.single;
  return "$digest";
}
