// Copyright (c) 2022, Pritam Hazra.

import 'package:encrypt/encrypt.dart';
import '../key/key_generator.dart';
import '../key/iv_generator.dart';

typedef GRAESMode = AESMode;

enum GRPadding { pkcs7, none }

String grEncrypt16AES(
    {required String input,
    required GRKey key,
    GRIV? iv,
    GRAESMode mode = GRAESMode.sic,
    GRPadding padding = GRPadding.pkcs7,
    int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  var aes = AES(key);
  switch (padding) {
    case GRPadding.pkcs7:
      aes = AES(key, mode: mode, padding: "PKCS7");
      break;
    case GRPadding.none:
      aes = AES(key, mode: mode, padding: null);
      break;
  }
  final encryptor = Encrypter(aes);
  var encrypted = encryptor.encrypt(input, iv: grIVFromLength(16));
  if (iv != null) {
    encrypted = encryptor.encrypt(input, iv: iv);
  }
  if (repeat == 0) {
    return encrypted.base16;
  }
  return grEncrypt16AES(
      input: encrypted.base16,
      key: key,
      iv: iv,
      mode: mode,
      padding: padding,
      repeat: repeat - 1);
}

String grEncrypt64AES(
    {required String input,
    required GRKey key,
    GRIV? iv,
    GRAESMode mode = GRAESMode.sic,
    GRPadding padding = GRPadding.pkcs7,
    int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  var aes = AES(key);
  switch (padding) {
    case GRPadding.pkcs7:
      aes = AES(key, mode: mode, padding: "PKCS7");
      break;
    case GRPadding.none:
      aes = AES(key, mode: mode, padding: null);
      break;
  }
  final encryptor = Encrypter(aes);
  var encrypted = encryptor.encrypt(input, iv: grIVFromLength(16));
  if (iv != null) {
    encrypted = encryptor.encrypt(input, iv: iv);
  }
  if (repeat == 0) {
    return encrypted.base64;
  }
  return grEncrypt64AES(
      input: encrypted.base64,
      key: key,
      iv: iv,
      mode: mode,
      padding: padding,
      repeat: repeat - 1);
}

dynamic grDecrypt16AES(
    {required String inputBase16,
    required GRKey key,
    GRIV? iv,
    GRAESMode mode = GRAESMode.sic,
    GRPadding padding = GRPadding.pkcs7,
    int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  var aes = AES(key);
  switch (padding) {
    case GRPadding.pkcs7:
      aes = AES(key, mode: mode, padding: "PKCS7");
      break;
    case GRPadding.none:
      aes = AES(key, mode: mode, padding: null);
      break;
  }
  final decryptor = Encrypter(aes);
  var decrypted = decryptor.decrypt16(inputBase16, iv: grIVFromLength(16));
  if (iv != null) {
    decrypted = decryptor.decrypt16(inputBase16, iv: iv);
  }
  if (repeat == 0) {
    return decrypted;
  }
  return grDecrypt16AES(
      inputBase16: decrypted,
      key: key,
      iv: iv,
      mode: mode,
      padding: padding,
      repeat: repeat - 1);
}

dynamic grDecrypt64AES(
    {required String inputBase64,
    required GRKey key,
    GRIV? iv,
    GRAESMode mode = GRAESMode.sic,
    GRPadding padding = GRPadding.pkcs7,
    int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  var aes = AES(key);
  switch (padding) {
    case GRPadding.pkcs7:
      aes = AES(key, mode: mode, padding: "PKCS7");
      break;
    case GRPadding.none:
      aes = AES(key, mode: mode, padding: null);
      break;
  }
  final decryptor = Encrypter(aes);
  var decrypted = decryptor.decrypt64(inputBase64, iv: grIVFromLength(16));
  if (iv != null) {
    decrypted = decryptor.decrypt64(inputBase64, iv: iv);
  }
  if (repeat == 0) {
    return decrypted;
  }
  return grDecrypt64AES(
      inputBase64: decrypted,
      key: key,
      iv: iv,
      mode: mode,
      padding: padding,
      repeat: repeat - 1);
}
