// Copyright (c) 2022, Pritam Hazra.

import 'dart:convert';
import 'package:encrypt/encrypt.dart';
import '../key/key_generator.dart';
import '../key/iv_generator.dart';

String grEncrypt16Fernet(
    {required String input, required GRKey key, GRIV? iv, int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  final b64key = GRKey.fromUtf8(base64Url.encode(key.bytes));
  final fernet = Fernet(b64key);
  final encryptor = Encrypter(fernet);
  var encrypted = encryptor.encrypt(input);
  if (iv != null) {
    encrypted = encryptor.encrypt(input, iv: iv);
  }
  if (repeat == 0) {
    return encrypted.base16;
  }
  return grEncrypt16Fernet(
      input: encrypted.base16, key: key, iv: iv, repeat: repeat - 1);
}

String grEncrypt64Fernet(
    {required String input, required GRKey key, GRIV? iv, int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  final b64key = GRKey.fromUtf8(base64Url.encode(key.bytes));
  final fernet = Fernet(b64key);
  final encryptor = Encrypter(fernet);
  var encrypted = encryptor.encrypt(input);
  if (iv != null) {
    encrypted = encryptor.encrypt(input, iv: iv);
  }
  if (repeat == 0) {
    return encrypted.base64;
  }
  return grEncrypt64Fernet(
      input: encrypted.base64, key: key, iv: iv, repeat: repeat - 1);
}

String grDecrypt16Fernet(
    {required String inputBase16,
    required GRKey key,
    GRIV? iv,
    int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  final b64key = GRKey.fromUtf8(base64Url.encode(key.bytes));
  final fernet = Fernet(b64key);
  final decryptor = Encrypter(fernet);
  var decrypted = decryptor.decrypt16(inputBase16);
  if (iv != null) {
    decrypted = decryptor.decrypt16(inputBase16, iv: iv);
  }
  if (repeat == 0) {
    return decrypted;
  }
  return grDecrypt16Fernet(
      inputBase16: decrypted, key: key, iv: iv, repeat: repeat - 1);
}

String grDecrypt64Fernet(
    {required String inputBase64,
    required GRKey key,
    GRIV? iv,
    int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  final b64key = GRKey.fromUtf8(base64Url.encode(key.bytes));
  final fernet = Fernet(b64key);
  final decryptor = Encrypter(fernet);
  var decrypted = decryptor.decrypt64(inputBase64);
  if (iv != null) {
    decrypted = decryptor.decrypt64(inputBase64, iv: iv);
  }
  if (repeat == 0) {
    return decrypted;
  }
  return grDecrypt64Fernet(
      inputBase64: decrypted, key: key, iv: iv, repeat: repeat - 1);
}
