// Copyright (c) 2022, Pritam Hazra.

import 'package:encrypt/encrypt.dart';
import 'package:encrypt/encrypt_io.dart';
import 'package:pointycastle/asymmetric/api.dart';

typedef GREncodingRSA = RSAEncoding;

Future<String> grEncrypt16RSA(
    {required String input,
    required String publicKeyPath,
    required String privateKeyPath,
    GREncodingRSA encoding = GREncodingRSA.PKCS1,
    int repeat = 0}) async {
  if (repeat < 0) {
    return "";
  }
  final publicKey = await parseKeyFromFile<RSAPublicKey>(publicKeyPath);
  final privateKey = await parseKeyFromFile<RSAPrivateKey>(publicKeyPath);
  final rsa =
      RSA(publicKey: publicKey, privateKey: privateKey, encoding: encoding);
  final encryptor = Encrypter(rsa);
  var encrypted = encryptor.encrypt(input);
  if (repeat == 0) {
    return encrypted.base16;
  }
  return grEncrypt16RSA(
      input: encrypted.base16,
      publicKeyPath: publicKeyPath,
      privateKeyPath: privateKeyPath,
      repeat: repeat - 1);
}

Future<String> grEncrypt64RSA(
    {required String input,
    required String publicKeyPath,
    required String privateKeyPath,
    GREncodingRSA encoding = GREncodingRSA.PKCS1,
    int repeat = 0}) async {
  if (repeat < 0) {
    return "";
  }
  final publicKey = await parseKeyFromFile<RSAPublicKey>(publicKeyPath);
  final privateKey = await parseKeyFromFile<RSAPrivateKey>(publicKeyPath);
  final rsa =
      RSA(publicKey: publicKey, privateKey: privateKey, encoding: encoding);
  final encryptor = Encrypter(rsa);
  var encrypted = encryptor.encrypt(input);
  if (repeat == 0) {
    return encrypted.base64;
  }
  return grEncrypt64RSA(
      input: encrypted.base64,
      publicKeyPath: publicKeyPath,
      privateKeyPath: privateKeyPath,
      repeat: repeat - 1);
}

Future<String> grDecrypt16RSA(
    {required String inputBase16,
    required String publicKeyPath,
    required String privateKeyPath,
    GREncodingRSA encoding = GREncodingRSA.PKCS1,
    int repeat = 0}) async {
  if (repeat < 0) {
    return "";
  }
  final publicKey = await parseKeyFromFile<RSAPublicKey>(publicKeyPath);
  final privateKey = await parseKeyFromFile<RSAPrivateKey>(privateKeyPath);
  final rsa =
      RSA(publicKey: publicKey, privateKey: privateKey, encoding: encoding);
  final decryptor = Encrypter(rsa);
  var decrypted = decryptor.decrypt16(inputBase16);
  if (repeat == 0) {
    return decrypted;
  }
  return grDecrypt16RSA(
      inputBase16: decrypted,
      publicKeyPath: publicKeyPath,
      privateKeyPath: privateKeyPath,
      repeat: repeat - 1);
}

Future<String> grDecrypt64RSA(
    {required String inputBase64,
    required String publicKeyPath,
    required String privateKeyPath,
    GREncodingRSA encoding = GREncodingRSA.PKCS1,
    int repeat = 0}) async {
  if (repeat < 0) {
    return "";
  }
  final publicKey = await parseKeyFromFile<RSAPublicKey>(publicKeyPath);
  final privateKey = await parseKeyFromFile<RSAPrivateKey>(privateKeyPath);
  final rsa =
      RSA(publicKey: publicKey, privateKey: privateKey, encoding: encoding);
  final decryptor = Encrypter(rsa);
  var decrypted = decryptor.decrypt64(inputBase64);
  if (repeat == 0) {
    return decrypted;
  }
  return grDecrypt64RSA(
      inputBase64: decrypted,
      publicKeyPath: publicKeyPath,
      privateKeyPath: privateKeyPath,
      repeat: repeat - 1);
}
