// Copyright (c) 2022, Pritam Hazra.

import 'package:encrypt/encrypt.dart';
import 'package:encrypt/encrypt_io.dart';
import 'package:pointycastle/asymmetric/api.dart';

Future<String> grSign16RSA(
    {required String input,
    required String publicKeyPath,
    required String privateKeyPath}) async {
  final publicKey = await parseKeyFromFile<RSAPublicKey>(publicKeyPath);
  final privateKey = await parseKeyFromFile<RSAPrivateKey>(privateKeyPath);
  final signer = Signer(RSASigner(RSASignDigest.SHA256,
      publicKey: publicKey, privateKey: privateKey));
  return signer.sign(input).base16;
}

Future<String> grSign64RSA(
    {required String input,
    required String publicKeyPath,
    required String privateKeyPath}) async {
  final publicKey = await parseKeyFromFile<RSAPublicKey>(publicKeyPath);
  final privateKey = await parseKeyFromFile<RSAPrivateKey>(privateKeyPath);
  final signer = Signer(RSASigner(RSASignDigest.SHA256,
      publicKey: publicKey, privateKey: privateKey));
  return signer.sign(input).base64;
}

Future<bool> grVerify16RSA(
    {required String input,
    required String signatureBase16,
    required String publicKeyPath,
    required String privateKeyPath}) async {
  final publicKey = await parseKeyFromFile<RSAPublicKey>(publicKeyPath);
  final privateKey = await parseKeyFromFile<RSAPrivateKey>(privateKeyPath);
  final signer = Signer(RSASigner(RSASignDigest.SHA256,
      publicKey: publicKey, privateKey: privateKey));
  return signer.verify16(input, signatureBase16);
}

Future<bool> grVerify64RSA(
    {required String input,
    required String signatureBase64,
    required String publicKeyPath,
    required String privateKeyPath}) async {
  final publicKey = await parseKeyFromFile<RSAPublicKey>(publicKeyPath);
  final privateKey = await parseKeyFromFile<RSAPrivateKey>(privateKeyPath);
  final signer = Signer(RSASigner(RSASignDigest.SHA256,
      publicKey: publicKey, privateKey: privateKey));
  return signer.verify64(input, signatureBase64);
}
