// Copyright (c) 2022, Pritam Hazra.

import 'dart:convert';
import 'package:encrypt/encrypt.dart';
import '../key/key_generator.dart';
import '../key/iv_generator.dart';

String grEncrypt16Salsa20(
    {required String input, required GRKey key, GRIV? iv, int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  final salsa = Salsa20(key);
  final encryptor = Encrypter(salsa);
  var encrypted = encryptor.encrypt(input, iv: grIVFromLength(8));
  if (iv != null) {
    encrypted = encryptor.encrypt(json.encode(input), iv: iv);
  }
  if (repeat == 0) {
    return encrypted.base16;
  }
  return grEncrypt16Salsa20(
      input: encrypted.base16, key: key, iv: iv, repeat: repeat - 1);
}

String grEncrypt64Salsa20(
    {required String input, required GRKey key, GRIV? iv, int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  final salsa = Salsa20(key);
  final encryptor = Encrypter(salsa);
  var encrypted = encryptor.encrypt(input, iv: grIVFromLength(8));
  if (iv != null) {
    encrypted = encryptor.encrypt(json.encode(input), iv: iv);
  }
  if (repeat == 0) {
    return encrypted.base64;
  }
  return grEncrypt64Salsa20(
      input: encrypted.base64, key: key, iv: iv, repeat: repeat - 1);
}

String grDecrypt16Salsa20(
    {required String inputBase16,
    required GRKey key,
    GRIV? iv,
    int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  final salsa = Salsa20(key);
  final decryptor = Encrypter(salsa);
  var decrypted = decryptor.decrypt16(inputBase16, iv: grIVFromLength(8));
  if (iv != null) {
    decrypted = decryptor.decrypt16(inputBase16, iv: iv);
  }
  if (repeat == 0) {
    return decrypted;
  }
  return grDecrypt16Salsa20(
      inputBase16: decrypted, key: key, iv: iv, repeat: repeat - 1);
}

String grDecrypt64Salsa20(
    {required String inputBase64,
    required GRKey key,
    GRIV? iv,
    int repeat = 0}) {
  if (repeat < 0) {
    return "";
  }
  final salsa = Salsa20(key);
  final decryptor = Encrypter(salsa);
  var decrypted = decryptor.decrypt64(inputBase64, iv: grIVFromLength(8));
  if (iv != null) {
    decrypted = decryptor.decrypt64(inputBase64, iv: iv);
  }
  if (repeat == 0) {
    return decrypted;
  }
  return grDecrypt64Salsa20(
      inputBase64: decrypted, key: key, iv: iv, repeat: repeat - 1);
}
